@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <stdlib.h>
 #include <stack>
@@ -24,21 +24,25 @@
 #include "meshGFaceOptimize.h"
 #include "meshGFaceBDS.h"
 #include "meshGRegion.h"
+#include "meshGRegionRelocateVertex.h"
+#include "meshGRegionLocalMeshMod.h"
 #include "BackgroundMesh.h"
 #include "BoundaryLayers.h"
 #include "HighOrder.h"
 #include "Generator.h"
 #include "Field.h"
 #include "Options.h"
-#include "simple3D.h"
-#include "yamakawa.h"
-#include "meshGRegionRelocateVertex.h"
-#include "pointInsertion.h"
 
 #if defined(_OPENMP)
 #include <omp.h>
 #endif
 
+#if defined(HAVE_DOMHEX)
+#include "simple3D.h"
+#include "yamakawa.h"
+#include "pointInsertion.h"
+#endif
+
 #if defined(HAVE_OPTHOM)
 #include "OptHomRun.h"
 #include "OptHomElastic.h"
@@ -49,7 +53,7 @@
 #include "PViewData.h"
 #endif
 
-class TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES {
+class EmbeddedCompatibilityTest {
 public:
   void operator()(GRegion *gr)
   {
@@ -280,9 +284,8 @@ static bool TooManyElements(GModel *m, int dim)
 {
   if(CTX::instance()->expertMode || !m->getNumVertices()) return false;
 
-  // try to detect obvious mistakes in characteristic lenghts (one of
-  // the most common cause for erroneous bug reports on the mailing
-  // list)
+  // try to detect obvious mistakes in characteristic lenghts (one of the most
+  // common cause for erroneous bug reports on the mailing list)
   double sumAllLc = 0.;
   for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it)
     sumAllLc +=
@@ -488,15 +491,13 @@ static void Mesh2D(GModel *m)
         if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 2D...");
       }
       if(!nPending) break;
-      // iter == 2 is for meshing re-parametrized surfaces
-      // after that, we serialize (self-intersections of 1D meshes are not thread safe)!
+      // iter == 2 is for meshing re-parametrized surfaces; after that, we
+      // serialize (self-intersections of 1D meshes are not thread safe)!
       if(nIter > 2) Msg::SetNumThreads(1);
       if(nIter++ > 10) break;
     }
   }
 
-  // collapseSmallEdges(*m);
-
   Msg::SetNumThreads(prevNumThreads);
 
   double t2 = Cpu();
@@ -549,60 +550,6 @@ FindConnectedRegions(const std::vector<GRegion *> &del,
             nbVolumes, connected.size());
 }
 
-template <class ITERATOR>
-void fillv_(std::multimap<MVertex *, MElement *> &vertexToElement,
-            ITERATOR it_beg, ITERATOR it_end)
-{
-  for(ITERATOR IT = it_beg; IT != it_end; ++IT) {
-    MElement *el = *IT;
-    for(std::size_t j = 0; j < el->getNumVertices(); j++) {
-      MVertex *e = el->getVertex(j);
-      vertexToElement.insert(std::make_pair(e, el));
-    }
-  }
-}
-
-int LaplaceSmoothing(GRegion *gr)
-{
-  std::multimap<MVertex *, MElement *> vertexToElement;
-  fillv_(vertexToElement, (gr)->tetrahedra.begin(), (gr)->tetrahedra.end());
-  fillv_(vertexToElement, (gr)->hexahedra.begin(), (gr)->hexahedra.end());
-  fillv_(vertexToElement, (gr)->prisms.begin(), (gr)->prisms.end());
-  fillv_(vertexToElement, (gr)->pyramids.begin(), (gr)->pyramids.end());
-  int N = 0;
-  for(unsigned int i = 0; i < gr->mesh_vertices.size(); i++) {
-    MVertex *v = gr->mesh_vertices[i];
-    std::multimap<MVertex *, MElement *>::iterator it =
-      vertexToElement.lower_bound(v);
-    std::multimap<MVertex *, MElement *>::iterator it_low = it;
-    std::multimap<MVertex *, MElement *>::iterator it_up =
-      vertexToElement.upper_bound(v);
-    double minQual = 1.e22;
-    double volTot = 0.0;
-    double xold = v->x(), yold = v->y(), zold = v->z();
-    SPoint3 pNew(0, 0, 0);
-    for(; it != it_up; ++it) {
-      minQual = std::min(minQual, it->second->minSICNShapeMeasure());
-      double vol = fabs(it->second->getVolume());
-      SPoint3 cog = it->second->barycenter();
-      pNew += cog * vol;
-      volTot += vol;
-    }
-    pNew *= (1. / volTot);
-    v->setXYZ(pNew.x(), pNew.y(), pNew.z());
-    double minQual2 = 1.e22;
-    for(it = it_low; it != it_up; ++it) {
-      minQual2 = std::min(minQual2, it->second->minSICNShapeMeasure());
-      if(minQual2 < minQual) {
-        v->setXYZ(xold, yold, zold);
-        break;
-      }
-    }
-    if(minQual < minQual2) N++;
-  }
-  return N;
-}
-
 // JFR : use hex-splitting to resolve non conformity
 //     : if howto == 1 ---> split hexes
 //     : if howto == 2 ---> create transition elements
@@ -704,7 +651,7 @@ bool MakeMeshConformal(GModel *gm, int howto)
             MFace(it->getVertex(1), it->getVertex(2), it->getVertex(3)));
         }
       }
-      // HEX IS ONLY SURROUNED BY COMPATIBLE ELEMENTS
+      // Hex is only surrounded by compatible elements
       if((int)faces.size() == e->getNumFaces()) {
         remainingHexes.push_back(e);
       }
@@ -745,7 +692,7 @@ bool MakeMeshConformal(GModel *gm, int howto)
             MFace(it->getVertex(1), it->getVertex(2), it->getVertex(3)));
         }
       }
-      // HEX IS ONLY SURROUNED BY COMPATIBLE ELEMENTS
+      // Hex is only surrounded by compatible elements
       if((int)faces.size() == e->getNumFaces()) {
         remainingPrisms.push_back(e);
       }
@@ -773,7 +720,8 @@ bool MakeMeshConformal(GModel *gm, int howto)
   return true;
 }
 
-void TestConformity(GModel *gm)
+#if defined(HAVE_DOMHEX)
+static void TestConformity(GModel *gm)
 {
   fs_cont search;
   buildFaceSearchStructure(gm, search);
@@ -809,6 +757,7 @@ void TestConformity(GModel *gm)
   else
     Msg::Error("Mesh is not conforming (%d hanging faces)!", count);
 }
+#endif
 
 static void Mesh3D(GModel *m)
 {
@@ -855,14 +804,18 @@ static void Mesh3D(GModel *m)
     }
   }
 
+#if defined(HAVE_DOMHEX)
   double time_recombination = 0., vol_element_recombination = 0.;
   double vol_hexa_recombination = 0.;
   int nb_elements_recombination = 0, nb_hexa_recombination = 0;
+#endif
 
   for(unsigned int i = 0; i < connected.size(); i++) {
     MeshDelaunayVolume(connected[i]);
 
-    // additional code for experimental hex mesh
+#if defined(HAVE_DOMHEX)
+    // additional code for experimental hex mesh - will eventually be replaced
+    // by new HXT-based code
     for(unsigned j = 0; j < connected[i].size(); j++) {
       GRegion *gr = connected[i][j];
       bool treat_region_ok = false;
@@ -884,9 +837,7 @@ static void Mesh3D(GModel *m)
           opt(gr);
         }
         double a = Cpu();
-
         // CTX::instance()->mesh.recombine3DLevel = 2;
-
         if(CTX::instance()->mesh.recombine3DLevel >= 0) {
           Recombinator rec;
           rec.execute(gr);
@@ -908,17 +859,13 @@ static void Mesh3D(GModel *m)
         nb_hexa_recombination += post.get_nb_hexahedra();
         vol_element_recombination += post.get_vol_elements();
         vol_hexa_recombination += post.get_vol_hexahedra();
-        // partial export
-        //        stringstream ss;
-        //        ss << "yamakawa_part_";
-        //        ss << gr->tag();
-        //        ss << ".msh";
-        //        export_gregion_mesh(gr, ss.str().c_str());
         time_recombination += (Cpu() - a);
       }
     }
+#endif
   }
 
+#if defined(HAVE_DOMHEX)
   if(CTX::instance()->mesh.recombine3DAll) {
     Msg::Info("RECOMBINATION timing:");
     Msg::Info(" --- CUMULATIVE TIME RECOMBINATION : %g s.", time_recombination);
@@ -930,18 +877,16 @@ static void Mesh3D(GModel *m)
     // MakeMeshConformal (m, 1);
     TestConformity(m);
   }
+#endif
 
   // ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
-  //  std::for_each(m->firstRegion(), m->lastRegion(),
-  //  optimizeMeshGRegionNetgen());
   if(Msg::GetVerbosity() > 98)
-    std::for_each(m->firstRegion(), m->lastRegion(),
-                  TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES());
+    std::for_each(m->firstRegion(), m->lastRegion(), EmbeddedCompatibilityTest());
 
   std::stringstream debugInfo;
-  debugInfo << "No tetrahedra in region ";
+  debugInfo << "No elements in region ";
   bool emptyRegionFound = false;
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it) {
     if((*it)->getNumMeshElements() == 0) {
@@ -972,12 +917,12 @@ void OptimizeMeshNetgen(GModel *m)
   double t1 = Cpu();
 
   std::for_each(m->firstRegion(), m->lastRegion(), optimizeMeshGRegionNetgen());
-  // Ensure that all volume Jacobians are positive
+
+  // ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
   if(Msg::GetVerbosity() > 98)
-    std::for_each(m->firstRegion(), m->lastRegion(),
-                  TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES());
+    std::for_each(m->firstRegion(), m->lastRegion(), EmbeddedCompatibilityTest());
 
   CTX::instance()->mesh.changed = ENT_ALL;
   double t2 = Cpu();
@@ -994,8 +939,7 @@ void OptimizeMesh(GModel *m)
   m->setAllVolumesPositive();
 
   if(Msg::GetVerbosity() > 98)
-    std::for_each(m->firstRegion(), m->lastRegion(),
-                  TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES());
+    std::for_each(m->firstRegion(), m->lastRegion(), EmbeddedCompatibilityTest());
 
   CTX::instance()->mesh.changed = ENT_ALL;
   double t2 = Cpu();
@@ -1111,9 +1055,6 @@ void GenerateMesh(GModel *m, int ask)
   else if(m->getMeshStatus() == 3 && CTX::instance()->mesh.algoSubdivide == 2)
     RefineMesh(m, CTX::instance()->mesh.secondOrderLinear, false, true);
 
-  // Compute homology if necessary
-  if(!Msg::GetErrorCount()) m->computeHomology();
-
   // Create high order elements
   if(m->getMeshStatus() && CTX::instance()->mesh.order > 1)
     SetOrderN(m, CTX::instance()->mesh.order,
@@ -1133,7 +1074,8 @@ void GenerateMesh(GModel *m, int ask)
       p.BARRIER_MAX = CTX::instance()->mesh.hoThresholdMax;
       p.dim = GModel::current()->getDim();
       p.optPrimSurfMesh = CTX::instance()->mesh.hoOptPrimSurfMesh;
-      HighOrderMeshOptimizer(GModel::current(), p);
+      // HighOrderMeshOptimizer(GModel::current(), p);
+      HighOrderMeshOptimizerNew(GModel::current(), p);
     }
 #else
     Msg::Error("High-order mesh optimization requires the OPTHOM module");
@@ -1145,6 +1087,9 @@ void GenerateMesh(GModel *m, int ask)
     m->renumberMeshElements();
   }
 
+  // Compute homology if necessary
+  if(!Msg::GetErrorCount()) m->computeHomology();
+
   Msg::Info("%d vertices %d elements", m->getNumMeshVertices(),
             m->getNumMeshElements());
 
