@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <stack>
 #include "GmshConfig.h"
@@ -176,9 +176,11 @@ void buildMeshGenerationDataStructures(
       it != vSizesMap.end(); ++it) {
     SPoint2 param;
     reparamMeshVertexOnFace(it->first, gf, param);
-    // Add size of background mesh to embedded vertices. For the other nodes, use the size in vSizesMap
+    // Add size of background mesh to embedded vertices. For the other nodes,
+    // use the size in vSizesMap
     const double lcBGM = (embeddedVertices.count(it->first) > 0) ?
-      BGM_MeshSize(gf, param[0], param[1], it->first->x(), it->first->y(), it->first->z()) : it->second;
+      BGM_MeshSize(gf, param[0], param[1], it->first->x(), it->first->y(),
+                   it->first->z()) : it->second;
     data.addVertex (it->first, param[0], param[1], it->second, lcBGM);
   }
   for(unsigned int i = 0; i < gf->triangles.size(); i++) {
